-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt
-- Author: Funky7Monkey

local C = {}
C.__index = C

function C:init()
  self.veloSmoother = newExponentialSmoothing(20, 1)
  self.lastDataPos = vec3()
  self.fov = self.fov or 25

  --- this marks it as base mode. It needs to set position, rotation and fov correctly
  self.baseMode = true
  -- add it to the default camera modes?
  self.register = true
end

function C:update(data)
  local ref  = vec3(data.veh:getNodePosition(self.refNodes.ref))
  local back = vec3(data.veh:getNodePosition(self.refNodes.back))

  local velo = self.veloSmoother:get((self.lastDataPos - data.pos):length())
  self.lastDataPos = data.pos

  local dir = (ref - back):normalized()
  local targetPos = data.pos + vec3(dir.x, dir.y, dir.z) * math.min(25, 10 * velo)
  self.fov = 10 + math.min(30, 15 * velo)
  local basPos = vec3(math.floor(targetPos['x'] / 100 + 0.5) * 100, math.floor(targetPos['y'] / 100 + 0.5) * 100, targetPos['z'] + 35)
  local camPos = basPos - vec3(0, 0, 0)
  local qdir = quatFromDir((targetPos - camPos):normalized())

  -- set the data, this needs to happen
  data.res.pos = camPos -- required, vec3()
  data.res.rot = qdir -- required, quat()
  data.res.fov = self.fov -- required
  data.res.targetPos = targetPos -- this is optional
  return true
end

-- DO NOT CHANGE CLASS IMPLEMENTATION BELOW

return function(...)
  local o = ... or {}
  setmetatable(o, C)
  o:init()
  return o
end
